#!/bin/bash -i


#!/bin/bash

#!/usr/bin/env bash
#!/bin/bash -i

sudo apt-get update
sudo apt-get --assume-yes upgrade
sudo apt-get --assume-yes install software-properties-common
sudo apt-get --assume-yes install jq
sudo apt-get --assume-yes install build-essential
sudo apt-get --assume-yes install linux-headers-$(uname -r)
wget https://repo.anaconda.com/miniconda/Miniconda3-py311_24.9.2-0-Linux-x86_64.sh
chmod +x Miniconda3-py311_24.9.2-0-Linux-x86_64.sh
./Miniconda3-py311_24.9.2-0-Linux-x86_64.sh -b -p $HOME/miniconda3
~/miniconda3/bin/conda init bash


source .bashrc
. .bashrc
. $HOME/.bashrc
exec bash
echo "done"